﻿using System.Collections.Generic;

namespace NUnit4PowerShell
{
    /// <summary>
    /// This class is a singleton providing the name of the file to execute
    /// It works like a unlimited queue (FIFO system)
    /// Once an item is Dequeued, the same item is enqueue at the queue end 
    /// </summary>
    public class PSFixtureProvider
    {
        /// <summary>
        /// This static property allow the user to launch 1 test (when he double clicks on the dll)
        /// When this option is true, the PSFixture first file is taken
        /// </summary>
        static public bool IsSuiteTest = false;

        #region Singleton region
        /// <summary>
        /// Private constructor for singleton
        /// </summary>
        private PSFixtureProvider()
        {
            if (IsSuiteTest)
            {
                // Look for the fixture files to provide
                foreach (string fixtureFile in Utils.GetPSFixtureFiles())
                {
                    _fixturesQueue.Enqueue(fixtureFile);
                }
            }
            else
            {
                _fixturesQueue.Enqueue(Utils.GetPSFixtureFiles()[0]);
            }
        }
        private static PSFixtureProvider instance;

        public static PSFixtureProvider Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new PSFixtureProvider();
                }
                return instance;
            }
        }
        #endregion

        private readonly Queue<string> _fixturesQueue = new Queue<string>();

        /// <summary>
        /// Provide the first item on the queue, remove it and reinject it at the end of the queue
        /// </summary>
        /// <returns></returns>
        private string Dequeue()
        {
            var item = _fixturesQueue.Dequeue();
            Enqueue(item);
            return item;
        }

        private void Enqueue(string item)
        {
            _fixturesQueue.Enqueue(item);
        }

        /// <summary>
        /// Contains the amount of item contained in the queue
        /// </summary>
        private int Count
        {
            get
            {
                return _fixturesQueue.Count;
            }
        }

        static public void AddOne(string item)
        {
            //Logger.Instance.Info("[FixtureProvider] AddOne " + item);
            PSFixtureProvider.Instance.Enqueue(item);
        }

        static public string GetOne()
        {
            if (PSFixtureProvider.Instance.Count == 0)
            {
                //Logger.Instance.Info("[FixtureProvider] GetOne NULL");
                return null;
            }
            var item = PSFixtureProvider.Instance.Dequeue();
            //Logger.Instance.Info("[FixtureProvider] GetOne " + item);
            return item;
        }

        static  public IEnumerable<string> Queue()
        {
            if (PSFixtureProvider.Instance.Count == 0)
            {
                //Logger.Instance.Info("[FixtureProvider] GetOne NULL");
                return null;
            }
            return PSFixtureProvider.Instance._fixturesQueue.ToArray();
        }
    }
}
